#include "fdbuf.h"
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

///////////////////////////////////////////////////////////////////////////////
// Class fdibuf
///////////////////////////////////////////////////////////////////////////////
fdibuf::fdibuf(int fdesc, bool dc, unsigned bufsize)
  : fdbuf(fdesc, dc, bufsize)
{
}

fdibuf::fdibuf(const char* filename, unsigned bufsize)
  : fdbuf(open(filename, O_RDONLY), true, bufsize)
{
  if(fd == -1) {
    flags = flag_error;
    errnum = errno;
  }
}

fdibuf::~fdibuf()
{
}

bool fdibuf::eof() const
{
  return (flags & flag_eof) && (start >= length);
}

bool fdibuf::operator!() const
{
  return eof() || error() || closed();
}

bool fdibuf::refill()
{
  if(flags)
    return false;
  if(start != 0) {
    if(start < length) {
      length -= start;
      memcpy(buf, buf+start, length);
    } else
      length = 0;
    start = 0;
  }
  unsigned oldlength = length;
  if(length < size) {
    ssize_t red = ::read(fd, buf+length, size-length);
    if(red == -1) {
      errnum = errno;
      flags |= flag_error;
    }
    else if(red == 0)
      flags |= flag_eof;
    else
      length += red;
  }
  return length > oldlength;
}

bool fdibuf::getline(mystring& out, char terminator)
{
  if(start >= length)
    refill();
  if(eof() || error())
    return false;
  out = "";
  while(!eof() && !error()) {
    char* ptr = buf+start;
    const char* end = (const char*)memchr(ptr, terminator, length-start);
    if(!end) {
      out += mystring(ptr, length-start);
      start = length;
      refill();
    }
    else {
      out += mystring(ptr, end-ptr);
      start += (end-ptr)+1;
      break;
    }
  }
  return true;
}

bool fdibuf::read(char* data, unsigned datalen)
{
  char* ptr = data;
  count = 0;
  while(datalen && !eof()) {
    if(start >= length)
      refill();
    unsigned len = length-start;
    if(len > datalen)
      len = datalen;
    memcpy(ptr, buf+start, len);
    start += len;
    datalen -= len;
    ptr += len;
    count += len;
  }
  return !datalen;
}

///////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////
fdibuf fin(0);
